@@ -9,9 +9,10 @@ from bec_lib import Alarms, BECService, MessageEndpoints, bec_logger, messages
 from bec_lib.connector import ConnectorBase
 from bec_lib.device import OnFailure
 from bec_lib.messages import BECStatus
-from ophyd import OphydObject, Staged
+from ophyd import Kind, OphydObject, Staged
 from ophyd.utils import errors as ophyd_errors
 
+from device_server.devices import rgetattr
 from device_server.devices.devicemanager import DeviceManagerDS
 from device_server.rpc_mixin import RPCMixin
 
@@ -99,7 +100,7 @@ class DeviceServer(RPCMixin, BECService):
     @staticmethod
     def consumer_interception_callback(msg, *, parent, **_kwargs) -> None:
         """callback for receiving scan modifications / interceptions"""
-        mvalue = messages.ScanQueueModificationMessage.loads(msg.value)
+        mvalue = msg.value
         if mvalue is None:
             logger.warning("Failed to parse scan queue modification message.")
             return
@@ -151,7 +152,7 @@ class DeviceServer(RPCMixin, BECService):
         """
         action = None
         try:
-            instructions = messages.DeviceInstructionMessage.loads(msg)
+            instructions = msg
             if not instructions.content["device"]:
                 return
             action = instructions.content["action"]
@@ -187,7 +188,7 @@ class DeviceServer(RPCMixin, BECService):
             self.connector.raise_alarm(
                 severity=Alarms.MAJOR,
                 source=instructions.content,
-                content=content,
+                msg=content,
                 alarm_type=limit_error.__class__.__name__,
                 metadata=instructions.metadata,
             )
@@ -201,7 +202,7 @@ class DeviceServer(RPCMixin, BECService):
                 self.connector.raise_alarm(
                     severity=Alarms.MAJOR,
                     source=instructions.content,
-                    content=content,
+                    msg=content,
                     alarm_type=exc.__class__.__name__,
                     metadata=instructions.metadata,
                 )
@@ -282,19 +283,20 @@ class DeviceServer(RPCMixin, BECService):
                 obj.obj.pre_scan()
             dev_msg = messages.DeviceReqStatusMessage(
                 device=dev, success=True, metadata=instr.metadata
-            ).dumps()
+            )
             self.producer.set_and_publish(MessageEndpoints.device_req_status(dev), dev_msg, pipe)
         pipe.execute()
 
     def _status_callback(self, status):
         pipe = self.producer.pipeline()
         if hasattr(status, "device"):
-            device_name = status.device.root.name
+            obj = status.device
         else:
-            device_name = status.obj.root.name
+            obj = status.obj
+        device_name = obj.root.name
         dev_msg = messages.DeviceReqStatusMessage(
             device=device_name, success=status.success, metadata=status.instruction.metadata
-        ).dumps()
+        )
         logger.debug(f"req status for device {device_name}: {status.success}")
         self.producer.set_and_publish(
             MessageEndpoints.device_req_status(device_name), dev_msg, pipe
@@ -307,9 +309,25 @@ class DeviceServer(RPCMixin, BECService):
                 pipe,
                 expire=18000,
             )
+        content = status.instruction.content
+        is_config_set = content["action"] == "set"
+        is_rpc_set = content["action"] == "rpc" and (".set" in content["parameter"]["func"])
 
+        if is_config_set or is_rpc_set:
+            if obj.kind == Kind.config:
+                self._update_read_configuration(obj, status.instruction.metadata, pipe)
+            elif obj.kind in [Kind.normal, Kind.hinted]:
+                self._read_device(status.instruction)
         pipe.execute()
 
+    def _update_read_configuration(self, obj: OphydObject, metadata: dict, pipe) -> None:
+        dev_config_msg = messages.DeviceMessage(
+            signals=obj.root.read_configuration(), metadata=metadata
+        )
+        self.producer.set_and_publish(
+            MessageEndpoints.device_read_configuration(obj.root.name), dev_config_msg, pipe
+        )
+
     def _read_device(self, instr: messages.DeviceInstructionMessage) -> None:
         # check performance -- we might have to change it to a background thread
         devices = instr.content["device"]
@@ -334,19 +352,17 @@ class DeviceServer(RPCMixin, BECService):
 
             self.producer.set_and_publish(
                 MessageEndpoints.device_read(device_root),
-                messages.DeviceMessage(signals=signals, metadata=metadata).dumps(),
+                messages.DeviceMessage(signals=signals, metadata=metadata),
                 pipe,
             )
             self.producer.set_and_publish(
                 MessageEndpoints.device_readback(device_root),
-                messages.DeviceMessage(signals=signals, metadata=metadata).dumps(),
+                messages.DeviceMessage(signals=signals, metadata=metadata),
                 pipe,
             )
             self.producer.set(
                 MessageEndpoints.device_status(device_root),
-                messages.DeviceStatusMessage(
-                    device=device_root, status=0, metadata=metadata
-                ).dumps(),
+                messages.DeviceStatusMessage(device=device_root, status=0, metadata=metadata),
                 pipe,
             )
         pipe.execute()
@@ -369,7 +385,7 @@ class DeviceServer(RPCMixin, BECService):
                 signals = self._retry_obj_method(dev, obj, "read_configuration", exc)
             self.producer.set_and_publish(
                 MessageEndpoints.device_read_configuration(dev),
-                messages.DeviceMessage(signals=signals, metadata=metadata).dumps(),
+                messages.DeviceMessage(signals=signals, metadata=metadata),
                 pipe,
             )
         pipe.execute()
@@ -383,7 +399,7 @@ class DeviceServer(RPCMixin, BECService):
             severity=Alarms.WARNING,
             alarm_type="Warning",
             source="DeviceServer",
-            content=f"Failed to run {method} on device {device}.",
+            msg=f"Failed to run {method} on device {device}.",
             metadata={},
         )
         device_root = device.split(".")[0]
@@ -400,13 +416,9 @@ class DeviceServer(RPCMixin, BECService):
                 f"Failed to run {method} on device {device_root}. Trying to load an old value."
             )
             if method == "read":
-                old_msg = messages.DeviceMessage.loads(
-                    self.producer.get(MessageEndpoints.device_read(device_root))
-                )
+                old_msg = self.producer.get(MessageEndpoints.device_read(device_root))
             elif method == "read_configuration":
-                old_msg = messages.DeviceMessage.loads(
-                    self.producer.get(MessageEndpoints.device_read_configuration(device_root))
-                )
+                old_msg = self.producer.get(MessageEndpoints.device_read_configuration(device_root))
             else:
                 raise ValueError(f"Unknown method {method}.")
             if not old_msg:
@@ -430,7 +442,7 @@ class DeviceServer(RPCMixin, BECService):
                 self.device_manager.devices[dev].obj.stage()
             self.producer.set(
                 MessageEndpoints.device_staged(dev),
-                messages.DeviceStatusMessage(device=dev, status=1, metadata=instr.metadata).dumps(),
+                messages.DeviceStatusMessage(device=dev, status=1, metadata=instr.metadata),
                 pipe,
             )
         pipe.execute()
@@ -451,7 +463,7 @@ class DeviceServer(RPCMixin, BECService):
                     logger.debug(f"Device {obj.name} was already unstaged.")
             self.producer.set(
                 MessageEndpoints.device_staged(dev),
-                messages.DeviceStatusMessage(device=dev, status=0, metadata=instr.metadata).dumps(),
+                messages.DeviceStatusMessage(device=dev, status=0, metadata=instr.metadata),
                 pipe,
             )
         pipe.execute()
