@@ -2,9 +2,8 @@ from __future__ import annotations
 
 import traceback
 from typing import TYPE_CHECKING
-from bec_lib import messages
 
-from bec_lib import BECStatus, DeviceConfigError, MessageEndpoints, bec_logger
+from bec_lib import BECStatus, DeviceConfigError, MessageEndpoints, bec_logger, messages
 
 if TYPE_CHECKING:
     from devicemanager import DeviceManagerDS
@@ -30,15 +29,14 @@ class ConfigUpdateHandler:
 
     @staticmethod
     def _device_config_callback(msg, *, parent, **_kwargs) -> None:
-        msg = messages.DeviceConfigMessage.loads(msg.value)
         logger.info(f"Received request: {msg}")
-        parent.parse_config_request(msg)
+        parent.parse_config_request(msg.value)
 
     def parse_config_request(self, msg: messages.DeviceConfigMessage) -> None:
         """Processes a config request. If successful, it emits a config reply
 
         Args:
-            msg (BMessage.DeviceConfigMessage): Config request
+            msg (BECMessage.DeviceConfigMessage): Config request
 
         """
         error_msg = ""
@@ -71,7 +69,7 @@ class ConfigUpdateHandler:
         )
         RID = metadata.get("RID")
         self.device_manager.producer.set(
-            MessageEndpoints.device_config_request_response(RID), msg.dumps(), expire=60
+            MessageEndpoints.device_config_request_response(RID), msg, expire=60
         )
 
     def _update_config(self, msg: messages.DeviceConfigMessage) -> None:
@@ -88,6 +86,16 @@ class ConfigUpdateHandler:
                     self.device_manager.update_config(device.obj, old_config)
                     raise DeviceConfigError(f"Error during object update. {exc}")
 
+                if "limits" in dev_config["deviceConfig"]:
+                    limits = {
+                        "low": device.obj.low_limit_travel.get(),
+                        "high": device.obj.high_limit_travel.get(),
+                    }
+                    self.device_manager.producer.set_and_publish(
+                        MessageEndpoints.device_limits(device.name),
+                        messages.DeviceMessage(signals=limits),
+                    )
+
             if "enabled" in dev_config:
                 device._config["enabled"] = dev_config["enabled"]
                 if dev_config["enabled"]:
