@@ -1,8 +1,12 @@
+from __future__ import annotations
+
 import inspect
+import sys
 import time
 import traceback
 from functools import reduce
 
+import numpy as np
 import ophyd
 import ophyd.sim as ops
 import ophyd_devices as opd
@@ -18,6 +22,7 @@ from bec_lib import (
 from bec_lib.connector import ConnectorBase
 from ophyd.ophydobj import OphydObject
 from ophyd.signal import EpicsSignalBase
+from typeguard import typechecked
 
 from device_server.devices.config_update_handler import ConfigUpdateHandler
 from device_server.devices.device_serializer import get_device_info
@@ -49,10 +54,8 @@ class DSDevice(DeviceBase):
 
     def initialize_device_buffer(self, producer):
         """initialize the device read and readback buffer on redis with a new reading"""
-        dev_msg = messages.DeviceMessage(signals=self.obj.read(), metadata={}).dumps()
-        dev_config_msg = messages.DeviceMessage(
-            signals=self.obj.read_configuration(), metadata={}
-        ).dumps()
+        dev_msg = messages.DeviceMessage(signals=self.obj.read(), metadata={})
+        dev_config_msg = messages.DeviceMessage(signals=self.obj.read_configuration(), metadata={})
         if hasattr(self.obj, "low_limit_travel") and hasattr(self.obj, "high_limit_travel"):
             limits = {
                 "low": self.obj.low_limit_travel.get(),
@@ -64,14 +67,12 @@ class DSDevice(DeviceBase):
         producer.set_and_publish(MessageEndpoints.device_readback(self.name), dev_msg, pipe=pipe)
         producer.set(topic=MessageEndpoints.device_read(self.name), msg=dev_msg, pipe=pipe)
         producer.set_and_publish(
-            MessageEndpoints.device_read_configuration(self.name),
-            dev_config_msg,
-            pipe=pipe,
+            MessageEndpoints.device_read_configuration(self.name), dev_config_msg, pipe=pipe
         )
         if limits is not None:
             producer.set_and_publish(
                 MessageEndpoints.device_limits(self.name),
-                messages.DeviceMessage(signals=limits).dumps(),
+                messages.DeviceMessage(signals=limits),
                 pipe=pipe,
             )
         pipe.execute()
@@ -109,11 +110,18 @@ class DeviceManagerDS(DeviceManagerBase):
         self.devices.flush()
         self._get_config()
 
-    def _get_device_class(self, dev_type):
-        """Return the class object from 'dev_type' string in the form '[module:][submodule:]class_name'
-
+    @staticmethod
+    def _get_device_class(dev_type: str) -> type:
+        """
+        Return the class object from 'dev_type' string in the form '[module:][submodule:]class_name'
         The class is looked after in ophyd devices[.module][.submodule] first, if it is not
         present plugin_devices, ophyd, ophyd_devices.sim are searched too
+
+        Args:
+            dev_type (str): device type string
+
+        Returns:
+            type: class object
         """
         submodule, _, class_name = dev_type.rpartition(":")
         if submodule:
@@ -157,8 +165,10 @@ class DeviceManagerDS(DeviceManagerBase):
                 # first handle the ophyd exceptions...
                 if config_key == "limits":
                     if hasattr(obj, "low_limit_travel") and hasattr(obj, "high_limit_travel"):
-                        obj.low_limit_travel.set(config_value[0])
-                        obj.high_limit_travel.set(config_value[1])
+                        low_limit_status = obj.low_limit_travel.set(config_value[0])
+                        high_limit_status = obj.high_limit_travel.set(config_value[1])
+                        low_limit_status.wait()
+                        high_limit_status.wait()
                         continue
                 if config_key == "labels":
                     if not config_value:
@@ -180,18 +190,20 @@ class DeviceManagerDS(DeviceManagerBase):
                 else:
                     setattr(obj, config_key, config_value)
 
-    def initialize_device(self, dev: dict) -> DSDevice:
+    @staticmethod
+    def construct_device_obj(dev: dict, device_manager: DeviceManagerDS) -> (OphydObject, dict):
         """
-        Prepares a device for later usage.
-        This includes inspecting the device class signature,
-        initializing the object, refreshing the device info and buffer,
-        as well as adding subscriptions.
+        Construct a device object from a device config dictionary.
+
+        Args:
+            dev (dict): device config dictionary
+            device_manager (DeviceManagerDS): device manager instance
+
+        Returns:
+            (OphydObject, dict): device object and updated config dictionary
         """
         name = dev.get("name")
-        enabled = dev.get("enabled")
-        read_only = dev.get("read_only", False)
-
-        dev_cls = self._get_device_class(dev["deviceClass"])
+        dev_cls = DeviceManagerDS._get_device_class(dev["deviceClass"])
         device_config = dev.get("deviceConfig")
         device_config = device_config if device_config is not None else {}
         config = device_config.copy()
@@ -215,14 +227,27 @@ class DeviceManagerDS(DeviceManagerBase):
         init_kwargs = {key: config.pop(key) for key in class_params_and_config_keys}
         device_access = config.pop("device_access", None)
         if device_access or (device_access is None and config.get("device_mapping")):
-            init_kwargs["device_manager"] = self
+            init_kwargs["device_manager"] = device_manager
 
         signature = inspect.signature(dev_cls)
         if "device_manager" in signature.parameters:
-            init_kwargs["device_manager"] = self
+            init_kwargs["device_manager"] = device_manager
 
         # initialize the device object
         obj = dev_cls(**init_kwargs)
+        return obj, config
+
+    def initialize_device(self, dev: dict) -> DSDevice:
+        """
+        Prepares a device for later usage.
+        This includes inspecting the device class signature,
+        initializing the object, refreshing the device info and buffer,
+        as well as adding subscriptions.
+        """
+        name = dev.get("name")
+        enabled = dev.get("enabled")
+
+        obj, config = self.construct_device_obj(dev, device_manager=self)
         self.update_config(obj, config)
 
         # refresh the device info
@@ -258,6 +283,8 @@ class DeviceManagerDS(DeviceManagerBase):
         elif "value" in obj.event_types:
             obj.subscribe(self._obj_callback_readback, run=opaas_obj.enabled)
 
+        if "monitor" in obj.event_types:
+            obj.subscribe(self._obj_callback_monitor, run=False)
         if "done_moving" in obj.event_types:
             obj.subscribe(self._obj_callback_done_moving, event_type="done_moving", run=False)
         if "flyer" in obj.event_types:
@@ -289,7 +316,7 @@ class DeviceManagerDS(DeviceManagerBase):
         obj.initialized = False
 
     @staticmethod
-    def connect_device(obj):
+    def connect_device(obj, wait_for_all=False):
         """establish a connection to a device"""
         if obj.connected:
             return
@@ -297,7 +324,10 @@ class DeviceManagerDS(DeviceManagerBase):
             obj.controller.on()
             return
         if hasattr(obj, "wait_for_connection"):
-            obj.wait_for_connection(timeout=10)
+            try:
+                obj.wait_for_connection(all_signals=wait_for_all, timeout=10)
+            except TypeError:
+                obj.wait_for_connection(timeout=10)
             return
 
         logger.error(
@@ -318,7 +348,7 @@ class DeviceManagerDS(DeviceManagerBase):
         interface = get_device_info(obj, {})
         self.producer.set(
             MessageEndpoints.device_info(obj.name),
-            messages.DeviceInfoMessage(device=obj.name, info=interface).dumps(),
+            messages.DeviceInfoMessage(device=obj.name, info=interface),
             pipe,
         )
 
@@ -328,24 +358,53 @@ class DeviceManagerDS(DeviceManagerBase):
         self.producer.delete(MessageEndpoints.device_read(obj.name), pipe)
         self.producer.delete(MessageEndpoints.device_info(obj.name), pipe)
 
-    def _obj_callback_readback(self, *_args, **kwargs):
-        obj = kwargs["obj"]
+    def _obj_callback_readback(self, *_args, obj: OphydObject, **kwargs):
         if obj.connected:
             name = obj.root.name
             signals = obj.read()
             metadata = self.devices.get(obj.root.name).metadata
-            dev_msg = messages.DeviceMessage(signals=signals, metadata=metadata).dumps()
+            dev_msg = messages.DeviceMessage(signals=signals, metadata=metadata)
             pipe = self.producer.pipeline()
             self.producer.set_and_publish(MessageEndpoints.device_readback(name), dev_msg, pipe)
             pipe.execute()
 
+    @typechecked
+    def _obj_callback_monitor(self, *_args, obj: OphydObject, value: np.ndarray, **kwargs):
+        """
+        Callback for ophyd monitor events. Sends the data to redis.
+        Introduces a check of the data size, and incoporates a limit which is defined in max_size (in MB)
+
+        Args:
+            obj (OphydObject): ophyd object
+            value (np.ndarray): data from ophyd device
+
+        """
+        # Convert sizes from bytes to MB
+        dsize = len(value.tobytes()) / 1e6
+        max_size = 100
+        if dsize > max_size:
+            logger.warning(
+                f"Data size of single message is too large to send, current max_size {max_size}."
+            )
+            return
+        if obj.connected:
+            name = obj.root.name
+            metadata = self.devices[name].metadata
+            msg = messages.DeviceMonitorMessage(device=name, data=value, metadata=metadata)
+            stream_msg = {"data": msg}
+            self.producer.xadd(
+                MessageEndpoints.device_monitor(name),
+                stream_msg,
+                max_size=min(100, int(max_size // dsize)),
+            )
+
     def _obj_callback_acq_done(self, *_args, **kwargs):
         device = kwargs["obj"].root.name
         status = 0
         metadata = self.devices[device].metadata
         self.producer.send(
             MessageEndpoints.device_status(device),
-            messages.DeviceStatusMessage(device=device, status=status, metadata=metadata).dumps(),
+            messages.DeviceStatusMessage(device=device, status=status, metadata=metadata),
         )
 
     def _obj_callback_done_moving(self, *args, **kwargs):
@@ -358,7 +417,7 @@ class DeviceManagerDS(DeviceManagerBase):
         metadata = self.devices[device].metadata
         self.producer.set(
             MessageEndpoints.device_status(kwargs["obj"].root.name),
-            messages.DeviceStatusMessage(device=device, status=status, metadata=metadata).dumps(),
+            messages.DeviceStatusMessage(device=device, status=status, metadata=metadata),
         )
 
     def _obj_flyer_callback(self, *_args, **kwargs):
@@ -385,16 +444,16 @@ class DeviceManagerDS(DeviceManagerBase):
             bundle.append(
                 messages.DeviceMessage(
                     signals={obj.name: signals}, metadata={"pointID": ii, **metadata}
-                ).dumps()
+                )
             )
         ds_obj.emitted_points[metadata["scanID"]] = max_points
         pipe = self.producer.pipeline()
-        self.producer.send(MessageEndpoints.device_read(obj.root.name), bundle.dumps(), pipe=pipe)
+        self.producer.send(MessageEndpoints.device_read(obj.root.name), bundle, pipe=pipe)
         msg = messages.DeviceStatusMessage(
             device=obj.root.name, status=max_points, metadata=metadata
         )
         self.producer.set_and_publish(
-            MessageEndpoints.device_progress(obj.root.name), msg.dumps(), pipe=pipe
+            MessageEndpoints.device_progress(obj.root.name), msg, pipe=pipe
         )
         pipe.execute()
 
@@ -403,4 +462,4 @@ class DeviceManagerDS(DeviceManagerBase):
         msg = messages.ProgressMessage(
             value=value, max_value=max_value, done=done, metadata=metadata
         )
-        self.producer.set_and_publish(MessageEndpoints.device_progress(obj.root.name), msg.dumps())
+        self.producer.set_and_publish(MessageEndpoints.device_progress(obj.root.name), msg)
