@@ -62,7 +62,7 @@ class RPCMixin:
                 severity=Alarms.WARNING,
                 alarm_type="TypeError",
                 source=instr_params,
-                content=f"Return value of rpc call {instr_params} is not serializable.",
+                msg=f"Return value of rpc call {instr_params} is not serializable.",
                 metadata={},
             )
         return res
@@ -74,7 +74,7 @@ class RPCMixin:
             MessageEndpoints.device_rpc(instr_params.get("rpc_id")),
             messages.DeviceRPCMessage(
                 device=device, return_val=res, out=result.getvalue(), success=True
-            ).dumps(),
+            ),
             expire=1800,
         )
 
@@ -116,10 +116,34 @@ class RPCMixin:
             ".read_configuration"
         ):
             return self._rpc_read_configuration_and_return(instr)
+        if instr_params.get("kwargs", {}).get("_set_property"):
+            sub_access = instr_params.get("func").split(".")
+            property_name = sub_access[-1]
+            if len(sub_access) > 1:
+                sub_access = sub_access[0:-1]
+            else:
+                sub_access = []
+            obj = self.device_manager.devices[device_root].obj
+            if sub_access:
+                obj = rgetattr(obj, ".".join(sub_access))
+            setattr(obj, property_name, instr_params.get("args")[0])
+            return None
 
         # handle other ophyd methods
         rpc_var = rgetattr(self.device_manager.devices[device_root].obj, instr_params.get("func"))
         res = self._get_result_from_rpc(rpc_var, instr_params)
+        if instr_params.get("func") == "put" or instr_params.get("func").endswith(".put"):
+            if instr_params.get("func") == "put":
+                obj = self.device_manager.devices[device_root].obj
+            else:
+                obj = rgetattr(
+                    self.device_manager.devices[device_root].obj,
+                    instr_params.get("func").split(".put")[0],
+                )
+            if obj.kind == ophyd.Kind.config:
+                return self._rpc_read_configuration_and_return(instr)
+            if obj.kind in [ophyd.Kind.normal, ophyd.Kind.hinted]:
+                return self._rpc_read_and_return(instr)
         if isinstance(res, ophyd.StatusBase):
             res.__dict__["instruction"] = instr
             res.add_callback(self._status_callback)
@@ -155,5 +179,5 @@ class RPCMixin:
             MessageEndpoints.device_rpc(instr_params.get("rpc_id")),
             messages.DeviceRPCMessage(
                 device=instr.content["device"], return_val=None, out=exc_formatted, success=False
-            ).dumps(),
+            ),
         )
